/***************************************************************************
                          speech.cpp  -  description
                             -------------------
    begin                : Son Sep 8 2002
    copyright            : (C) 2002 by Gunnar Schmi Dt
    email                : kmouth@schmi-dt.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// $Id: speech.cpp,v 1.5 2004/03/22 03:08:11 cramblitt Exp $

#include "speech.h"
#include <qfile.h>
#include <qstring.h>
#include <qvaluelist.h>
#include <qstringlist.h>
#include <qregexp.h>
#include <qtextcodec.h>
#include <kdebug.h>
#include <qvaluestack.h>

Speech::Speech() {
#if KDE_VERSION >= 310
   process.setUseShell(true);
#endif
}

Speech::~Speech() {
}

void Speech::speak(QString command, bool stdIn, QString text,
                   int encoding, QTextCodec *codec, QString language)
{
   text = text.stripWhiteSpace();
   if (text.length () > 0) {
      // 1. prepare the text:
      // 1.a) encode the text
      QTextStream ts (encText, IO_WriteOnly);
      if (encoding == Local)
         ts.setEncoding (QTextStream::Locale);
      else if (encoding == Latin1)
         ts.setEncoding (QTextStream::Latin1);
      else if (encoding == Unicode)
         ts.setEncoding (QTextStream::Unicode);
      else
         ts.setCodec (codec);
      ts << text;

      // 1.b) quote the text as one parameter
      QString escText = KShellProcess::quote(encText);

      // 1.c) create a temporary file for the text
      tempFile.setAutoDelete(true);
      QTextStream* fs = tempFile.textStream();
      if (encoding == Local)
         fs->setEncoding (QTextStream::Locale);
      else if (encoding == Latin1)
         fs->setEncoding (QTextStream::Latin1);
      else if (encoding == Unicode)
         fs->setEncoding (QTextStream::Unicode);
      else
         fs->setCodec (codec);
      *fs << text;
      *fs << endl;
      QString filename = tempFile.file()->name();
      tempFile.close();

      // FIXME: This is just a quick hack to get the standard input to work
      //        with the synchronous approach of kttsd.
      if (stdIn)
         command = "cat %f | (" + command + ")";
      
      // 2. replace variables with values
	QValueStack<bool> stack;
	bool issinglequote=false;
	bool isdoublequote=false;
	int noreplace=0;
	QRegExp re_noquote("(\"|'|\\\\|`|\\$\\(|\\$\\{|\\(|\\{|\\)|\\}|%%|%t|%f|%l)");
	QRegExp re_singlequote("('|%%|%t|%f|%l)");
	QRegExp re_doublequote("(\"|\\\\|`|\\$\\(|\\$\\{|%%|%t|%f|%l)");

	for	( int i = re_noquote.search(command);
		i != -1;
		i = (issinglequote?re_singlequote.search(command,i)
			:isdoublequote?re_doublequote.search(command,i)
			:re_noquote.search(command,i))
	)
	{
		if ((command[i]=='(') || (command[i]=='{')) // (...) or {...}
		{
			// assert(isdoublequote == false)
			stack.push(isdoublequote);
			if (noreplace > 0)
				// count nested braces when within ${...}
				noreplace++;
			i++;
		}
		else if (command[i]=='$')
		{
			stack.push(isdoublequote);
			isdoublequote = false;
			if ((noreplace > 0) || (command[i+1]=='{'))
				// count nested braces when within ${...}
				noreplace++;
			i+=2;
		}
		else if ((command[i]==')') || (command[i]=='}'))
			// $(...) or (...) or ${...} or {...}
		{
			if (!stack.isEmpty())
				isdoublequote = stack.pop();
			else
				qWarning("Parse error.");
			if (noreplace > 0)
				// count nested braces when within ${...}
				noreplace--;
			i++;
		}
		else if (command[i]=='\'')
		{
			issinglequote=!issinglequote;
			i++;
		}
		else if (command[i]=='"')
		{
			isdoublequote=!isdoublequote;
			i++;
		}
		else if (command[i]=='\\')
			i+=2;
		else if (command[i]=='`')
		{
			// Replace all `...` with safer $(...)
			command.replace (i, 1, "$(");
			QRegExp re_backticks("(`|\\\\`|\\\\\\\\|\\\\\\$)");
			for (	int i2=re_backticks.search(command,i+2);
				i2!=-1;
				i2=re_backticks.search(command,i2)
			)
			{
				if (command[i2] == '`')
				{
					command.replace (i2, 1, ")");
					i2=command.length(); // leave loop
				}
				else
				{	// remove backslash and ignore following character
					command.remove (i2, 1);
					i2++;
				}
			}
			// Leave i unchanged! We need to process "$("
		}
		else if (noreplace == 0) // do not replace macros within ${...}
		{
			QString match, v;

			// get match
			if (issinglequote)
				match=re_singlequote.cap();
			else if (isdoublequote)
				match=re_doublequote.cap();
			else
				match=re_noquote.cap();

			// substitue %variables
			if (match=="%%")
				v="%";
			else if (match=="%t")
				v=escText;
			else if (match=="%f")
				v=filename;
			else if (match=="%l")
				v=language;

			// %variable inside of a quote?
			if (isdoublequote)
				v='"'+v+'"';
			else if (issinglequote)
				v="'"+v+"'";

			command.replace (i, match.length(), v);
			i+=v.length();
		}
		else
		{
			if (issinglequote)
				i+=re_singlequote.matchedLength();
			else if (isdoublequote)
				i+=re_doublequote.matchedLength();
			else
				i+=re_noquote.matchedLength();
		}
	}

      // 3. create a new process
      kdDebug() << "Speech::Speak: running command: " << command << endl;
      process << command;
      connect(&process, SIGNAL(processExited(KProcess *)), this, SLOT(processExited(KProcess *)));
      connect(&process, SIGNAL(wroteStdin(KProcess *)), this, SLOT(wroteStdin(KProcess *)));
      connect(&process, SIGNAL(receivedStdout(KProcess *, char *, int)), this, SLOT(receivedStdout(KProcess *, char *, int)));
      connect(&process, SIGNAL(receivedStderr(KProcess *, char *, int)), this, SLOT(receivedStderr(KProcess *, char *, int)));

      // 4. start the process
      
      // Not needed with the quick hack above. We will use it again once kttsd
      // supports asynchronous mode.
      //if (stdIn) {
      //   process.start(KProcess::NotifyOnExit, KProcess::All);
      //   if (encText.size() > 0)
      //      process.writeStdin(encText, encText.size());
      //   else
      //      process.closeStdin();
      //}
      //else
         process.start(KProcess::Block, KProcess::AllOutput);
   }
}

void Speech::receivedStdout (KProcess *, char *buffer, int buflen) {
   kdDebug() << QString::fromLatin1(buffer, buflen) << endl;
}
void Speech::receivedStderr (KProcess *, char *buffer, int buflen) {
   kdDebug() << QString::fromLatin1(buffer, buflen) << endl;
}

void Speech::wroteStdin(KProcess *) {
   process.closeStdin();
}

void Speech::processExited(KProcess *) {
   this->deleteLater();
}

#include "speech.moc"
